'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _lodash = require('lodash.assign');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.compact');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.find');

var _lodash6 = _interopRequireDefault(_lodash5);

var _lodash7 = require('lodash.isstring');

var _lodash8 = _interopRequireDefault(_lodash7);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var defaultOptions = {
	blockPrefix: '',
	elementPrefix: '__',
	modifierPrefix: '--'
};

var opts = undefined,
    t = undefined;

exports.default = function (_ref) {
	var options = _ref.options;
	var types = _ref.types;

	opts = (0, _lodash2.default)({}, defaultOptions, options);
	t = types;

	return {
		visitor: {
			CallExpression: function CallExpression(node) {
				if (t.isReturnStatement(node.parent)) {
					walkCallExpressions(null, node.node);
				}
			}
		}
	};
};

function walkCallExpressions(ancestorBlock, node) {
	/* istanbul ignore if */
	if (!isReactCreateElementExpression(node)) {
		return;
	}

	var _node$arguments = _toArray(node.arguments);

	var type = _node$arguments[0];
	var props = _node$arguments[1];

	var children = _node$arguments.slice(2);

	if (!t.isObjectExpression(props)) {
		children.forEach(walkCallExpressions.bind(this, ancestorBlock));
		return;
	}

	if (t.isIdentifier(type)) {
		var _getBlockAndElement = getBlockAndElement(props);

		var _block = _getBlockAndElement.block;
		var _element = _getBlockAndElement.element;

		if (_element && !_block) {
			props.properties.unshift(new t.ObjectProperty(new t.Identifier('block'), new t.StringLiteral(ancestorBlock)));
		}
		children.forEach(walkCallExpressions.bind(this, _block || ancestorBlock));
		return;
	}

	var _consumeBEMProperties = consumeBEMProperties(props);

	var block = _consumeBEMProperties.block;
	var element = _consumeBEMProperties.element;
	var modifiers = _consumeBEMProperties.modifiers;

	block = block || ancestorBlock;

	children.forEach(walkCallExpressions.bind(this, block));

	if (!validateBEMAttributes({ block: block, element: element, modifiers: modifiers })) {
		return;
	}

	var properties = props.properties;

	assignClassName({ properties: properties, block: block, element: element, modifiers: modifiers });
}

function isReactCreateElementExpression(node) {
	var callee = node.callee;
	/* istanbul ignore if */

	if (!t.isMemberExpression(callee)) {
		return false;
	}
	/* istanbul ignore if */
	if (callee.object.name !== 'React') {
		return false;
	}
	/* istanbul ignore if */
	if (callee.property.name !== 'createElement') {
		return false;
	}
	return true;
}

function getBlockAndElement(obj) {
	var block = undefined,
	    element = undefined,
	    modifiers = undefined;
	obj.properties.forEach(function (_ref2, index) {
		var key = _ref2.key;
		var value = _ref2.value;

		if (key.name === 'element') {
			element = resolveTokenValue(value);
			return;
		}
		/* istanbul ignore else */
		if (key.name === 'block') {
			block = resolveTokenValue(value);
			return;
		}
	});
	return { block: block, element: element, modifiers: modifiers };
}

function consumeBEMProperties(obj) {
	var block = undefined,
	    element = undefined,
	    modifiers = undefined;
	obj.properties.forEach(function (_ref3, index) {
		var key = _ref3.key;
		var value = _ref3.value;

		if (key.name === 'element') {
			element = consumeProperty({ value: value, index: index });
			return;
		}
		if (key.name === 'modifiers') {
			modifiers = consumeProperty({ value: value, index: index });
			return;
		}
		/* istanbul ignore else */
		if (key.name === 'block') {
			block = consumeProperty({ value: value, index: index });
			return;
		}
	});
	obj.properties = (0, _lodash4.default)(obj.properties);
	return { block: block, element: element, modifiers: modifiers };

	function consumeProperty(_ref4) {
		var value = _ref4.value;
		var index = _ref4.index;

		delete obj.properties[index];
		return resolveTokenValue(value);
	}
}

function resolveTokenValue(token) {
	if (t.isStringLiteral(token) || t.isBooleanLiteral(token)) {
		return token.value;
	}
	if (t.isIdentifier(token)) {
		return '{' + token.name + '}';
	}
	return token;
}

function validateBEMAttributes(_ref5) {
	var block = _ref5.block;
	var element = _ref5.element;
	var modifiers = _ref5.modifiers;

	if (block) {
		return true;
	}
	if (element) {
		throw new Error('BEM element must have an ancestor block');
	}
	if (modifiers) {
		throw new Error('BEM modifiers must be attached to a block or an element');
	}
	return false;
}

function assignClassName(_ref6) {
	var properties = _ref6.properties;
	var block = _ref6.block;
	var element = _ref6.element;
	var modifiers = _ref6.modifiers;

	var prefix = '' + opts.blockPrefix + block;
	if (element) {
		prefix += '' + opts.elementPrefix + element;
	}
	var className = buildModifiers(prefix, modifiers);

	var classNameProp = (0, _lodash6.default)(properties, function (prop) {
		return prop.key.name === 'className';
	});
	if (!classNameProp) {
		properties.push(buildClassName());
		return;
	}

	var value = classNameProp.value;

	if (t.isStringLiteral(value)) {
		value.value = className + ' ' + value.value;
		return;
	}

	throw new Error('Unsupported className for BEM block or element');

	function buildClassName() {
		return new t.ObjectProperty(new t.Identifier('className'), (0, _lodash8.default)(className) ? new t.StringLiteral(className) : new t.CallExpression(new t.Identifier('classnames'), className));
	}
}

function buildModifiers(prefix, modifiers) {
	if (!modifiers) {
		return prefix;
	}
	if ((0, _lodash8.default)(modifiers)) {
		return [prefix].concat(modifiers.split(/\s+/g).map(function (modifier) {
			return '' + prefix + opts.modifierPrefix + modifier;
		})).join(' ');
	}
	if (t.isObjectExpression(modifiers)) {
		return [new t.StringLiteral(prefix)].concat((0, _lodash4.default)(modifiers.properties.map(function (_ref7) {
			var key = _ref7.key;
			var value = _ref7.value;

			key = new t.StringLiteral('' + prefix + opts.modifierPrefix + key.name);
			if (t.isBooleanLiteral(value)) {
				if (value.value === false) {
					return false;
				}
				return key;
			}
			return new t.ObjectExpression([new t.ObjectProperty(key, value)]);
		})));
	}
	throw new Error('Unsupported value for BEM modifiers');
}